@@ -47,7 +47,6 @@
 doc_comment::doctest!("../../README.md");
 
 mod client;
-mod client_pool;
 /// A pool of clients, to handle concurrent checks
 pub mod collector;
 mod helpers;
@@ -73,9 +72,15 @@ use ring as _; // required for apple silicon
 
 #[doc(inline)]
 pub use crate::{
-    client::{check, ClientBuilder},
-    client_pool::ClientPool,
+    // Constants get exposed so that the CLI can use the same defaults as the library
+    client::{
+        check, Client, ClientBuilder, DEFAULT_MAX_REDIRECTS, DEFAULT_MAX_RETRIES,
+        DEFAULT_RETRY_WAIT_TIME, DEFAULT_TIMEOUT, DEFAULT_USER_AGENT,
+    },
     collector::Collector,
     filter::{Excludes, Filter, Includes},
-    types::{Base, ErrorKind, Input, Request, Response, ResponseBody, Result, Status, Uri},
+    types::{
+        Base, ErrorKind, FileType, Input, InputContent, InputSource, Request, Response,
+        ResponseBody, Result, Status, Uri,
+    },
 };
